﻿

# ===== REGULAR IMPORTS =====
import os
import re
import datetime
import configobj
# ===== REGULAR IMPORTS END =====


# ===== EVENTSCRIPTS IMPORTS =====
import es
import cmdlib
import services
import gamethread
import installlib
# ===== EVENTSCRIPTS IMPORTS END =====


addonpath, addonname = os.path.split(__file__)
addonname_pure, addonname_filetype = addonname.rsplit('.', 1)
# es.dbgmsg(0, 'Addonname: %s' % addonname)
# es.dbgmsg(0, 'Addonpath: %s' % addonpath)
configpath = os.path.join(addonpath, 'config.ini')



info = es.AddonInfo()
info.name = 'AdminRestart'
info.esamname = 'adminrestart'
info.author = 'Brainsucker'
info.version = '1.0.2 rev14'
info.description = 'Allows Admins to restart the Server by Chat or Consolecommand'
info.url = 'http://addons.eventscripts.com/addons/view/%s' % info.esamname

# This regex allows to specify an exact time of the day, when the server will be restarted.
time_regex = re.compile(r'(?P<hour>\d{,2})\:(?P<minute>\d{,2})')


def total_seconds(td):
	return (td.days * 86400) + td.seconds
	
# if not hasattr(datetime.timedelta, 'total_seconds'):
	# datetime.timedelta.total_seconds = total_seconds


class DelayedRestart(object):
	
	logpath = os.path.join(addonpath, 'restarts.txt')
	# Should be enough, to calculate these values only 1 time.
	base_announce_times = range(1800, 3600, 600) + range(600, 1800, 300) + range(60, 600, 60) + range(10, 60, 10) + range(1, 6)
	def __init__(self):
		self.restart_running = False
		self.delay_time = 60
		
		# Add here some custom functions. These functions will be executed right before the Server will shut down.
		self.preRestartFunctions = []
		
	def announceMSG(self):
		if self.restart_running:
			rwt = newcfg.as_int('restart_warn_time')
			if (rwt > 0) and (self.remaining_seconds <= rwt):
				x = datetime.datetime.today().replace(microsecond=0)
				td_remaining = self.restart_tp - x
				td_remaining_str = str(td_remaining) # .rsplit('.', 1)[0]
				if 3600 <= self.remaining_seconds:
					message_text = '#default[#greenWARNING#default] Server will SHUTDOWN and RESTART at #green%s#default. Time remaining: #green%s #default[HH:MM:SS].' % (self.restart_tp.strftime('%x %X'), td_remaining_str)
					
				elif  300 <= self.remaining_seconds < 3600:
					message_text = '#default[#greenWARNING#default] Server will SHUTDOWN and RESTART soon. Time remaining: #green%s #default[HH:MM:SS].' % td_remaining_str
					
				else:
					message_text = '#default[#greenWARNING#default] Server will SHUTDOWN and RESTART in #green%i seconds#default.' % self.remaining_seconds
				
				
				es.msg('#multi', message_text)
	
	def startCountdown(self):
		if not self.restart_running:
			self.remaining_seconds = self.delay_time
			
			self.announce_delay_times = range(3600, self.delay_time, 1800) + filter(lambda x: x < self.delay_time, self.base_announce_times)
			self.restart_running = True
			gamethread.delayed(1, self.loopCounter)
			n = datetime.datetime.today().replace(microsecond=0)
			self.restart_tp = n + datetime.timedelta(seconds=self.delay_time)
			
	def cancelRestart(self):
		self.restart_running = False
		gamethread.cancelDelayed('adminrestart_restart_loop')
	
	def loopCounter(self):
		if self.restart_running:
			gamethread.delayedname(1, 'adminrestart_restart_loop', self.loopCounter)
			
			
		current_time = datetime.datetime.today().replace(microsecond=0)
		self.remaining_time_timedelta = self.restart_tp - current_time
		self.remaining_seconds = total_seconds(self.remaining_time_timedelta)
		
		if self.remaining_seconds == 0:
			self.processRestart()
			
		if self.remaining_seconds in self.announce_delay_times:
			self.announceMSG()
		
	def processRestart(self):
		for f in self.preRestartFunctions:
			if callable(f):
				try:
					f()
				except:
					pass
		# Time for restart
		es.server.queuecmd('_restart')
		
		
	def logText(self, text):
		d = datetime.datetime.today()
		logPrefix = d.strftime('[%x %X]')
		logfile = open(self.logpath, 'ab')
		logfile.write('%s %s\n' % (logPrefix, text))
		logfile.close()
		
restarter = DelayedRestart()

def preRestart():
	restarter.logText('Server has been restarted')
	
restarter.preRestartFunctions.append(preRestart)


def handleArgs(args, src_userid=None):
	td = datetime.datetime.today().replace(microsecond=0)
	def_restarttime = newcfg.as_int('default_restart_time')
	min_restart_time = newcfg.as_int('min_restart_time')
	max_restart_time = newcfg.as_int('max_restart_time')
	args_str_lo = args.lower()
	if args_str_lo in ['cancel', 'stop']:
		if restarter.restart_running:
			restarter.cancelRestart()
			es.msg('#multi', '#default[SERVER RESTART] has been canceled!')
			text = 'Server restart has been canceled by %s'
			if src_userid:
				name = es.getplayername(src_userid)
				steamid = es.getplayersteamid(src_userid)
				text = text % ('%s [%s]' % (name, steamid))
			else:
				text = text % 'RemoteCONsole'
			restarter.logText(text)
		else:
			tt = 'There is currently no running Server restart'
			if src_userid:
				es.tell(src_userid, '#multi', '#default' + tt)
			else:
				es.dbgmsg(0, tt)
				# print tt
				# es.server.queuecmd('echo ' + tt)
			
		return
		
	elif args_str_lo == 'info':
		if restarter.restart_running:
			tt = 'Server will restart at $color%s$color. ($color%i seconds $colorremaining)' % (restarter.restart_tp.strftime('%x %X'), restarter.remaining_seconds)
			if src_userid:
				text = formatColors(tt, ['#lightgreen', '#default','#green', '#default'])
				es.tell(src_userid, '#multi', '#default' + text)
				
			else:
				text = formatColors(tt)
				es.dbgmsg(0, text)
				# print text
				# es.server.queuecmd('echo ' + text)
				
			
		else:
			tt = 'There is currently no running Server restart'
			if src_userid:
				es.tell(src_userid, '#multi', '#default' + tt)
			else:
				es.dbgmsg(0, tt)
				# print tt
				# es.server.queuecmd('echo ' + tt)
			
		return
		
		
	args_valid = False
	
	restart_time = td + datetime.timedelta(seconds=def_restarttime)
	time_remaining_seconds = def_restarttime
	time_match = time_regex.match(args)
	if time_match:
		restart_hour = int(time_match.group('hour'))
		restart_minute = int(time_match.group('minute'))
		restart_time = td.replace(hour=restart_hour, minute=restart_minute, second=0)
		# Check, if the restart_time is already passed:
		if restart_time < td:
			# Just restart the Server 1 day later
			restart_time += datetime.timedelta(days=1)
			
		time_remaining_timedelta = restart_time - td
		time_remaining_seconds = total_seconds(time_remaining_timedelta)
		
		args_valid = True
			
	if args.isdigit():
		args_int = int(args)
		restart_time = td + datetime.timedelta(seconds=args_int)
		
		time_remaining_seconds = args_int
		
		args_valid = True
				

	if args_valid:
		if not restarter.restart_running:
			if min_restart_time <= time_remaining_seconds <= max_restart_time:
				text_part1 = 'Server restart has been initialized by %s. '
				text_part2 = 'Server will restart at %s (%i seconds until restart)' % (restart_time.strftime('%x %X'), time_remaining_seconds)
				if src_userid:
					name = es.getplayername(src_userid)
					steamid = es.getplayersteamid(src_userid)
					text_part1 = text_part1 % ('%s [%s]' % (name, steamid))
				else:
					text_part1 = text_part1 % 'RemoteCONsole'
				text = text_part1 + text_part2
				restarter.logText(text)
				restarter.delay_time = time_remaining_seconds
				restarter.startCountdown()
				
				return
			
			else:
				tt = 'Delay time must be BETWEEN %i and %i Seconds.' % (min_restart_time, max_restart_time)
				if src_userid:
					actionBlocked(tt, src_userid)
				else:
					actionBlocked(tt)
			
		else:
			t = 'There is already a running restart'
			if src_userid:
				actionBlocked(t, src_userid)
			else:
				actionBlocked(t)
			
		return
		
		
	error_txt = 'The Arguments you have entered are not valid.'
	if src_userid:
		es.tell(src_userid, '#multi', '#default' + error_txt)
	else:
		es.dbgmsg(0, error_txt)
		# print error_txt
		# es.server.queuecmd('echo ' + error_txt)

def restart(userid, args):
	steamid = es.getplayersteamid(userid)
	
	args_len = len(args)
	args_str = ''
	if args_len > 0:
		args_str = str(args)# IMHO, this should return an empty string ( '' ) if there are no arguments... :/
		
	args_str_lo = args_str.lower()
		
		
		
	if args_str_lo in ['cancel', 'stop']:
		if isAuthed(steamid, 'cancel_restart'):
			handleArgs(args_str, userid)
			return
			
		else:
			noAuth(userid)
		
		
	elif args_str_lo == 'info':
		if isAuthed(steamid, 'restart_info'):
			handleArgs(args_str, userid)
			return
				
		else:
			noAuth(userid)
	
	if isAuthed(steamid, 'restart_server'):
		handleArgs(args_str, userid)
		return
		
	else:
		noAuth(userid)
		
def restart_rcon(args):
	args_len = len(args)
	args_str = ''
	if args_len > 0:
		args_str = str(args)
	
	handleArgs(args_str)
		
def noAuth(userid):
	es.tell(userid, '#multi' ,'#defaultYou are #greenNOT #defaultallowed to use this command!')
	
def formatColors(text, colors=None):
	c_regex = re.compile(r'\$color', re.I)
	if colors:
		for c in colors:
			text = c_regex.sub(c, text, 1)
			
	else:
		text = c_regex.sub('', text)
		
	return text
	
def actionBlocked(reason, userid=None):
	t = '#defaultThis action has been blocked. Reason: $color%s' % reason
	if userid:
		text = formatColors(t, ['#green'])
		es.tell(userid, '#multi', text)
	else:
		text = formatColors(t)
		es.dbgmsg(0, text)
		# print t
		# es.server.queuecmd('echo ' + t)
		
def load():
	global isAuthed, newcfg
	if not es.exists('saycommand', '!restart'):
		cmdlib.registerSayCommand('!restart', restart, 'Allows Admins to restart the Server by Chat')
	if not es.exists('clientcommand', 'restart_server'):
		cmdlib.registerClientCommand('restart_server', restart, 'Allows Admins to restart the Server by Console')
	if not es.exists('command', 'ar_restart'):
		cmdlib.registerServerCommand('ar_restart', restart_rcon, 'Allows Admins to restart the Server by Remote Console')
		
		
	# Maybe the server could not have any internet connection.
	try:
		installer = installlib.getInstaller(info.esamname)
		update_result = installer.update()
	except:
		es.dbgmsg(1, '[%s] Updatecheck failed.' % info.name)
	
	es.ServerVar(info.name, info.version).makepublic()
	
	if services.isRegistered('auth'):
		auth = services.use('auth')
		auth.registerCapability('restart_server', auth.ROOT)
		auth.registerCapability('cancel_restart', auth.ROOT)
		auth.registerCapability('restart_info', auth.ADMIN)
		
		isAuthed = lambda x, y: auth.isIdAuthorized(x, y)
		
	else:
		isAuthed = lambda x, y: False
		es.log('[AdminRestart] WARNING: No AuthProvider has been found. Please check out the following website: http://www.eventscripts.com/pages/Basic_auth')
	
	
	# This will keep the config file up to date. YAY! :)
	
	current_conf = configobj.ConfigObj(configpath)
	
	newcfg = configobj.ConfigObj()
	newcfg.indent_type = '\t'
	newcfg.filename = current_conf.filename
	
	newcfg.initial_comment = ['Configuration file for AdminRestart Version: %s' % info.version, 'Addon written by Brainsucker.', 'If you founds any bugs or you got any recommendations, visit the following website:', info.url, 'If you want to reset this config to default settings, just delete it and restart your server (or reload this Addon by using "es_reload adminrestart")!', 'Thanks for using this Addon and have fun!', '']
	
	newcfg['default_restart_time'] = 30
	newcfg.comments['default_restart_time'] = ['Sets the default delay time (in seconds), which is used, when an admin uses the restart command without any parameters']
	
	newcfg['min_restart_time'] = 10
	newcfg.comments['min_restart_time'] = ['Minimum delay time (in seconds) after using the restart command until the restart is processed.']
	
	newcfg['max_restart_time'] = 86400
	newcfg.comments['max_restart_time'] = ['Maximum delay time (in seconds) after using the restart command until the restart is processed.']
	
	newcfg['restart_warn_time'] = 18000
	newcfg.comments['restart_warn_time'] = ['Sets the time before the restart, when the server starts announcing the restart.']
	
	newcfg['instant_restart_on_empty_server'] = True
	newcfg.comments['instant_restart_on_empty_server'] = ['This option will instantly restart the server, if there is any restart running and every (human) player left the server.', 'Set to False to disable']
	
	newcfg.merge(current_conf)
	
	
	newcfg.write()
		
		
def player_disconnect(ev):
	instant_restart = newcfg.as_bool('instant_restart_on_empty_server')
	if restarter.restart_running:
		if instant_restart:
			human_players = filter(lambda temp: not es.isbot(temp), es.getUseridList())
			if len(human_players) == 0:
				restarter.processRestart()

def unload():
	if restarter.restart_running:
		restarter.logText('Restart has been canceled due the Addon has been unloaded.')
		restarter.cancelRestart()
		
	if es.exists('command', 'ar_restart'):
		cmdlib.unregisterServerCommand('ar_restart')
	if es.exists('saycommand', '!restart'):
		cmdlib.unregisterSayCommand('!restart')
	if es.exists('clientcommand', 'restart_server'):
		cmdlib.unregisterClientCommand('restart_server')
	
